/*
 * deviced
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <unistd.h>
#include <bundle.h>
#include <eventsystem.h>

#include "core/log.h"
#include "core/device-notifier.h"
#include "core/common.h"
#include "display/poll.h"
#include "core/edbus-handler.h"
#include "shared/deviced-systemd.h"

#define SIGNAL_BOOTING_DONE		"BootingDone"

#define SYSTEMD_DBUS_SIGNAL_SYSTEM_STARTUP_FINISHED "StartupFinished"
#define SYSTEMD_DBUS_SIGNAL_USER_STARTUP_FINISHED   "UserSessionStartupFinished"
#define SYSTEMD_DBUS_METHOD_SYSTEM_STATE     "SystemState"
#define SYSTEMD_STATE_RUNNING                "running"
#define SYSTEMD_STATE_DEGRADED               "degraded"

int booting_finished(void)
{
	char *state;
	int ret;
	size_t len;

	ret = deviced_systemd_get_manager_property_as_string(
			SYSTEMD_DBUS_IFACE_MANAGER,
			SYSTEMD_DBUS_METHOD_SYSTEM_STATE,
			&state);
	if (ret < 0) {
		_E("Failed to get System State (%d)", ret);
		return ret;
	}

	_I("System State: (%s)", state);

	len = strlen(state) + 1;
	if (!strncmp(state, SYSTEMD_STATE_RUNNING, len) ||
		!strncmp(state, SYSTEMD_STATE_DEGRADED, len))
		ret = 1;
	else
		ret = 0;

	free(state);

	return ret;
}

void remove_booting_done_handler(void *data)
{
	unregister_edbus_signal_handler(SYSTEMD_DBUS_PATH,
			SYSTEMD_DBUS_IFACE_MANAGER,
			SYSTEMD_DBUS_SIGNAL_SYSTEM_STARTUP_FINISHED);
	unregister_edbus_signal_handler(SYSTEMD_DBUS_PATH,
			SYSTEMD_DBUS_IFACE_MANAGER,
			SYSTEMD_DBUS_SIGNAL_USER_STARTUP_FINISHED);
}

static void booting_done_received(void *data, DBusMessage *msg)
{
	static int system_done = 0;
	static int user_done = 0;

	if (dbus_message_is_signal(msg, SYSTEMD_DBUS_IFACE_MANAGER,
				SYSTEMD_DBUS_SIGNAL_SYSTEM_STARTUP_FINISHED)) {
		if (system_done)
			return;

		system_done = booting_finished();
		if (system_done == 0) {
			_E("System Session is not ready yet");
			return;
		}
		_I("System Session is Ready");
		device_notify(DEVICE_NOTIFIER_BOOTING_DONE, &system_done);
	}

	else if (dbus_message_is_signal(msg, SYSTEMD_DBUS_IFACE_MANAGER,
				SYSTEMD_DBUS_SIGNAL_USER_STARTUP_FINISHED)) {
		if (user_done)
			return;
		user_done = 1;
		_I("User Session is Ready");
	}

	if (!system_done || !user_done)
		return;

	remove_booting_done_handler(NULL);

	_I("real booting done, unlock LCD_OFF");
	pm_unlock_internal(INTERNAL_LOCK_BOOTING, LCD_OFF, PM_SLEEP_MARGIN);

	_I("signal booting done");
	broadcast_edbus_signal(DEVICED_PATH_CORE,
			DEVICED_INTERFACE_CORE,
			SIGNAL_BOOTING_DONE,
			NULL, NULL);
}

void add_booting_done_handler(void *data)
{
	/* System Session is loaded completely */
	register_edbus_signal_handler(SYSTEMD_DBUS_PATH,
		    SYSTEMD_DBUS_IFACE_MANAGER,
		    SYSTEMD_DBUS_SIGNAL_SYSTEM_STARTUP_FINISHED,
		    booting_done_received);
	/* User Session is loaded completely */
	register_edbus_signal_handler(SYSTEMD_DBUS_PATH,
		    SYSTEMD_DBUS_IFACE_MANAGER,
		    SYSTEMD_DBUS_SIGNAL_USER_STARTUP_FINISHED,
		    booting_done_received);
}
